#!/usr/local/bin/perl5 -w

# Copyright (c) 1999 Gabriel Deal. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


use lib '../lib';

use CGI qw(start_table end_table);
use IO::File;
use strict;
use Rball;
use Globals;


my $debug = undef;

my $g_page_name = 'UWIRL New Member Sign-Up Form';

#####################33

my $c = new CGI;

$| = 1;	# get the header out immediately
			# so nothing printed to STDOUT will cause a "server error"
print $c->header,
		$c->start_html({-Title => $g_page_name});
$| = 0;

my @errors = ();
if($c->param() and ! (@errors = check_input($c))) 
{
	accept_user($c);
} 
else
{
	if(@errors > 0)
	{
		print $c->h1('Errors in input'),
			'Please correct the errors noted below and re-submit.',
			$c->blockquote(join($c->br, @errors));
	}
	print_prompt($c);
}

print $Globals::footer
	 . $c->end_html;

exit 0;

############################################

sub print_prompt
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	print $c->h1($g_page_name),
		"Welcome to the Ladder!	 We're glad to have you on board. ",
		"Fill in the information below, and I'll see you on the courts!",
		$c->p,
		$c->strong("Please do not sign up if you don't have access to the IMA at the University of Washington"),
		" -- you will not be able to participate unless you have access to the IMA.   UW students, faculty, and staff can all get access to the IMA, I don't know of any others who can use the IMA.",
		$c->p,
		$c->startform({-Method => 'POST', -Action => $c->self_url}),
		'Full Name: ',
		$c->textfield(-name => "name", -size => 30, -maxlength => 50),
		$c->br,
		'Email Address: ',
		$c->textfield(-name => "email", -size => 30, -maxlength => 100),
		$c->br,
		'Home Page (optional): ',
		$c->textfield(-name => "url", -size => 60, -maxlength => 150, -default => "http://"),
		$c->br,
		'Phone (optional): ',
		$c->textfield(-name => "phone", -size => 15, -maxlength => 15),
		$c->br,
		'Affiliation: ',
		$c->popup_menu(-name => "affiliation", -Values => ['student', 'grad student', 'faculty', 'staff', 'other']),
		$c->br,
		'Department or Program (optional): ',
		$c->textfield(-name => "department", -size => 30, -maxlength => 50),
		$c->br,
		'Estimated skill level: ',
		$c->radio_group(
			-name => 'ladder',
			-default => 4,
			-Values => [1, 2, 3, 4],
			-labels => {
				1 => 'Advanced ', 
				2 => 'Advanced intermediate ',  
				3 => 'Intermediate', 
				4 => 'Beginner'
			}
		),
		$c->br,
		'Choose a Password: ',
		$c->password_field(-name => 'passwd', -size => 20, -maxlength => 20),
		$c->br,
		'Retype your Password: ',
		$c->password_field(-name => 'passwd2', -size => 20, -maxlength => 20),
		$c->br,
		$c->submit(-name => 'Join the Ladder!'),
		$c->endform;
}

sub check_input 
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my @errors = ();

	push @errors, "Missing Name.\n" if ! $c->param('name');
	push @errors, "Missing Email Address.\n"	if ! $c->param('email');
	push @errors, "Missing Password.\n" if ! $c->param('passwd');
	push @errors, "You didn't type the same password twice.\n" if ! $c->param('passwd2') or  $c->param('passwd') ne $c->param('passwd2');

	my $rball = new Rball "r";
	my %info = $rball->get_info($c->param('email'));
	if(keys(%info) > 0)
	{
		push @errors, $c->param('email') . " is already in use.\n";
	}

	return @errors;
}

sub accept_user 
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my $rball = new Rball;
	my(@ladders) = @Globals::ladder_names;
	@ladders = @Globals::ladder_names; # die stupid -w

#	$p{'department'} = $c->param('department');
#	$p{'phone'} = $c->param('phone');

	my $ladder = $c->param('ladder');
	$ladder = 3 if $ladder == 4;

	$rball->create_user($c->param('email'),
		# map({ $_ => $c->param($_) } $c->param),
		'name' => $c->param('name'),
		'affiliation' => $c->param('affiliation'),
		'department' => $c->param('department'),
		'url' => ($c->param('url') eq "http://") ? "" : $c->param('url'),
		'passwd' => $c->param('passwd'),
		'ladder' => $ladder,
		'phone' => $c->param('phone'),
		'status' => 1,
		'wins' => 0,
		'losses' => 0,
		'defaults' => 0,
		'rank' => 1 + $rball->max($ladder),
		'create-info' => "www:" . time,
		'modify-info' => "www:" . time
	);

	$rball->create_description(time,
		'event' => $c->param('email') . " joined the ladder",
		'description' => $c->param('description') ? $c->param('description') : "",
		'create-info' => "www:" . time,
		'modify-info' => "www:" . time
	);

	print $c->h1("Welcome!")
		.  "You have been added to the bottom of "
		. $ladders[$ladder - 1]
		. ".  To get started playing, go to the "
		. $c->a({Href => "display_rankings.cgi"}, "rankings page")
		. " and email a challenge to either of the two people directly "
		. "above you on the ladder.   You may also want to read the "
		. $c->a({Href => "../html/tourney-rules.html"}, "tournament rules.")
		. $c->p
		. "Good luck!";


	Misc::debug_end($debug) if defined $debug and $debug;
}

