#!/usr/local/bin/perl5 -w

# Copyright (c) 1999 Gabriel Deal. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


use lib '../lib';

use CGI qw(start_table end_table);
use strict;
use Rball;
use Globals;

my $debug = undef;



# some sub's use this still...
my $g_page_name = 'UWIRL Account Deletion Form';

&main($g_page_name);
exit 0;


############################################


sub main ($$)
{
	my $page_name = shift;

	my $c = new CGI;

	$| = 1;	# get the header out immediately
				# so nothing printed to STDOUT will cause a "server error"
	print $c->header({-expires => "now"}),
			$c->start_html({-Title => $page_name});
	$| = 0;

	my $mode = $c->param('mode');
	$c->delete('mode');
	my @errors = ();
	my %info;
	if(defined $mode and $mode eq "initial")
	{	
		process_login_input($c, $Global::this_dir, \%info, \@errors);
		if(@errors > 0)
		{
			print $c->h1('Errors in input'),
				'Please correct the errors noted below and re-submit.',
				$c->blockquote(join($c->br, @errors));
			print_login_prompt($c);
		}
		else
		{
			print_delete_prompt($c, %info);
		}
	} 
	elsif(defined $mode and $mode eq "delete")
	{
		do_delete($c->param('email'), $c);
	}
	else
	{
		print_login_prompt($c);
	}
	
	print $Globals::footer
		 . $c->end_html;
	
}

sub do_delete
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ >= 2 or Misc::bad_args("2", scalar @_);
	my $email = shift;
	my $c = shift;

	my $rball = new Rball;

	$rball->user_delete($email);

	print $c->h1($g_page_name)
		. $email 
		. ", you have been removed from the ladder.   I hope you have enjoyed yourself.   I've saved your records, so if you ever wish to rejoin, you may want to email the maintainer and see if you can get your old records restored.";

	Misc::debug_end($debug) if defined $debug and $debug;
}

sub print_delete_prompt
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ >= 1 or Misc::bad_args("1+", scalar @_);
	my $c = shift;
	my %info = @_;

	print $c->h1($g_page_name)
		. "Clicking on the button below will remove you from the ladder."
		. $c->startform({-Method => 'POST', -Action => $c->self_url})
		. $c->submit(-name => "Delete " . $c->param('email'))
		. $c->hidden(
			-name => 'email',
			-Value => $c->param('email')
		)
		. $c->hidden(
			-name => 'mode',
			-Value => 'delete'
		)
		. $c->endform;
}


sub print_login_prompt
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my $rball = new Rball;
	my @users = sort $rball->get_email;

	print $c->h1($g_page_name)
		. $c->startform({-Method => 'POST', -Action => $c->self_url})
		. 'Email Address: '
		. $c->popup_menu(
			-name => "email", 
			-Values => [ @users ],
		)
		. $c->br, "\n"
		. 'Password: '
		. $c->password_field(-name => 'passwd', -size => 20, -maxlength => 20)
		. $c->br, "\n"
		. $c->hidden(-name => 'mode', -value => 'initial')
		. $c->submit
		. $c->endform;
}



sub process_login_input
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 4 or Misc::bad_args(4, scalar @_);
	my $c = shift;
	my $dir = shift;
	my $info_ref = shift;
	my $error_ref = shift;

	push @$error_ref, "Email address not specified.\n" if ! $c->param('email');
	push @$error_ref, "Password not specified.\n" if ! $c->param('passwd');
	return if @$error_ref > 0;

	my $rball = new Rball;
	%{ $info_ref } = $rball->get_info($c->param('email'));

	if(keys %{ $info_ref } == 0)
	{
		push @$error_ref, "Invalid account: " . $c->param('email') . ".\n";
	}
	elsif(${ $info_ref }{'passwd'} ne $c->param('passwd'))
	{
		push @$error_ref, "Invalid password.\n";
	}

	return;
}

sub check_input 
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my @errors = ();

	push @errors, "Missing Name.\n" if ! $c->param('name');
	push @errors, "Missing Email Address.\n"	if ! $c->param('email');
	push @errors, "Missing Password.\n" if ! $c->param('passwd');
	push @errors, "You didn't type the same password twice.\n" if ! $c->param('passwd2') or  $c->param('passwd') ne $c->param('passwd2');

	return @errors;
}

