#!/usr/local/bin/perl5 -w

# Copyright (c) 1999 Gabriel Deal. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


use lib '../lib';

use CGI qw(start_table end_table);
use strict;
use Rball;
use Globals;

my $debug = undef;
my $g_page_name = 'UWIRL Match Report Form';

##########

my $dummy;
$dummy = $Globals::diff_limit;
$dummy = $Globals::maintainer_email;
$dummy = $Globals::ladder_names;

my $c = new CGI;

&main();

###############

sub main
{
	$| = 1;	# get the header out immediately
				# so nothing printed to STDOUT will cause a "server error"
	print $c->header({-expires => "now"}),
	$c->start_html({-Title => $g_page_name});
	$| = 0;

	my $mode = $c->param('mode');
	$mode = defined $mode ? $mode : "";
	$c->delete('mode');
	my @errors = ();
	if($mode eq "report" and ! (@errors = &check_input($c)))
	{
		&make_changes($c);
	}
	else
	{
		if(@errors > 0)
		{
			print $c->h1('Errors in input'),
				'Please correct the errors noted below and re-submit.',
				$c->blockquote(join($c->br, @errors));
		}
		&print_prompt($c);	
	}

	print $Globals::footer;
	print $c->end_html;
}


sub make_changes
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my $forfeit = $c->param('forfeit') =~ /forfeit/ ? 1 : 0;

	my $rball = new Rball;

	my $winner_email = $c->param('winner');
	my $loser_email = $c->param('loser');

	my %winner = $rball->get_info($winner_email);
	my %loser = $rball->get_info($loser_email);
	my $winner_ladder = $winner{'ladder'};
	my $loser_ladder = $loser{'ladder'};
	my $num_forfeit = $loser{'forfeit'};
	my $wins = $winner{'wins'};
	my $losses = $loser{'losses'};
	my $winner_rank = $winner{'rank'};
	my $loser_rank = $loser{'rank'};

	$num_forfeit = 0 if $num_forfeit !~ /^\s*\d+\s*$/;

	print $c->h1("UWIRL Match Report Results"),
		$c->br,
		"Congratulations on your victory!";

	my $diff = $rball->rank_difference($winner_email, $loser_email);
	if($diff < 0)
	{
		$rball->modify_user($winner_email, $winner_email,
			'ladder' => $loser_ladder,
			'rank' => $loser_rank,
			'wins' => $wins + 1
		);
		$rball->modify_user($loser_email, $loser_email,
			'ladder' => $winner_ladder,
			'rank' => $winner_rank,
			'forfeit' => $c->param('forfeit') =~ /forfeit/i ? ++$num_forfeit : 0,
			'losses' => $losses + 1
		);
		$rball->make_ladders();
		print "  You are now $loser_rank on $Globals::ladder_names[$loser_ladder - 1].\n";
	}
	else
	{
		$rball->modify_user(
			$winner_email,
			$winner_email,
			'wins' => $wins + 1
		);
		# only reset forfeits for loser!
		$rball->modify_user(
			$loser_email,
			$loser_email,
			'forfeit' => $c->param('forfeit') =~ /forfeit/i ? ++$num_forfeit : 0,
			'losses' => $losses + 1
		);
	}

	if($num_forfeit > 3)
	{
		system(qq!echo "$loser_email has forfeited $num_forfeit times." | /bin/mailx -s 'UWIRL forfeits' $Globals::maintainer_email!);

	}

	srand(time ^ $$);
	my $num = int(rand $#Globals::victory_phrases);
	my $victory_phrase;
	my $str1 = "#$winner_rank ranked $winner_email from "
			. $Globals::ladder_names[$winner_ladder - 1];
	my $str2 = "#$loser_rank ranked $loser_email from "
			. "$Globals::ladder_names[$loser_ladder - 1]";
	my $str;
	if($c->param('forfeit') =~ /forfeit/i)
	{
		$str = "$str2 forfeited to $str1";
	}
	else
	{
		$victory_phrase = $Globals::victory_phrases[int(rand(scalar @Globals::victory_phrases) - 1)];	
		$str = "$str1 $victory_phrase $str2";
	}

	$rball->create_description(time,
		'event' => $str,
		'description' => $c->param('description'),
		'create-info' => "www:" . time,
		'modify-info' => "www:" . time
	);

	Misc::debug_end($debug) if defined $debug and $debug;
}

sub check_input
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my @errors = ();
	my $winner = $c->param('winner');
	my $loser = $c->param('loser');
	push @errors, "Did not specify the winner.\n" if ! $winner;
	push @errors, "Did not specify the loser.\n" if ! $loser;

	my $rball = new Rball "r";
	if($winner and $loser)
	{
		push @errors, "Winner and loser are same person.\n" if $winner eq $loser;
		my $diff = $rball->rank_difference($winner, $loser);
		push @errors, "Winner and loser are not within $Globals::diff_limit spots of each other on the ladder, the match is invalid (see UWIRL rules).\n" if abs($diff) > 2;
	}
	push @errors, "Did not specify password." if ! $c->param('passwd');
	if(@errors == 0)
	{
		my %info = $rball->get_info($c->param('winner'));
		if(keys(%info) == 0)
		{
			push @errors, "No info in DB for " . $c->param('winner') . "(bug).\n";
		}
		else
		{
			my $passwd = $info{'passwd'};
			push @errors, "Invalid password.\n" if $passwd ne $c->param('passwd');
		}
	}

	Misc::debug_start($debug) if defined $debug and $debug;
	return @errors;
}


sub print_prompt 
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my $rball = new Rball "r";
	my @users = sort $rball->get_email;

	print $c->h1($g_page_name),
		 $c->startform({-Method => 'POST', -Action => $c->self_url});
	my $text = 
			$c->Tr(
				$c->td('The Winner (me)'),
				$c->td('The Loser (my victim)')
			)
		.	$c->Tr(
				$c->th(
					$c->popup_menu(
						-name => 'winner',
						-Values => [ @users ]
								)
				),
				$c->th(
					$c->popup_menu(
					-name => 'loser',
					-Values => [ @users ]
				)
			)
		);

	print $c->table($text),
		$c->p,
		$c->radio_group(
			-name => 'forfeit', 
			-Values => [' We played. ', ' I won by forfeit. '],
			-forfeit => 'We played.'
		);
  
	$text = 
		$c->Tr(
			$c->td(
				$c->b('Match Description')
				. $c->br
				. "To be published in The Court Report.  Tell us the scores and highlights, in your own creative words. Don't be shy!"
			),
			$c->td(
				$c->textarea(
						-name => 'description',
						-rows => 10,
						-columns => 50
				)
			)
		);
	$text .= "\n";

	print $c->table($text),
		$c->b('My Password: '), 
		$c->password_field(
			-name => 'passwd',
			-size => 20,
			-maxlength => 20
		),
		$c->p,
		$c->hidden(
			-name => 'mode',
			-Value => 'report'
		),
		$c->submit(-name=>'Submit Report'),
		$c->endform;

	$rball = undef;
}


