#!/usr/local/bin/perl5 -w

# Copyright (c) 1999 Gabriel Deal. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


use lib '../lib';

use CGI qw(start_table end_table);
use strict;
use Rball;
use Globals;

my $debug = undef;



# some sub's use this still...
my $g_page_name = 'UWIRL Account Modification Form';

&main($g_page_name);
exit 0;


############################################


sub main ($$)
{
	my $page_name = shift;

	my $c = new CGI;

	$| = 1;	# get the header out immediately
				# so nothing printed to STDOUT will cause a "server error"
	print $c->header({-expires => "now"}),
			$c->start_html({-Title => $page_name});
	$| = 0;

	my $mode = $c->param('mode');
	$c->delete('mode');
	my @errors = ();
	my %info;
	if(defined $mode and $mode eq "initial")
	{	
		process_login_input($c, $Global::this_dir, \%info, \@errors);
		if(@errors > 0)
		{
			print $c->h1('Errors in input'),
				'Please correct the errors noted below and re-submit.',
				$c->blockquote(join($c->br, @errors));
			print_login_prompt($c);
		}
		else
		{
			print_change_prompt($c, %info);
		}
	} 
	elsif(defined $mode and $mode eq "changes")
	{
		process_change_input($c, $Global::this_dir, \%info, \@errors);
		if(@errors > 0)
		{
			print $c->h1('Errors in input'),
				'Please correct the errors noted below and re-submit.',
				$c->blockquote(join($c->br, @errors));
			print_change_prompt($c, %info);
		}
	}
	else
	{
		print_login_prompt($c);
	}
	
	print $Globals::footer
		 . $c->end_html;
	
}

sub process_change_input
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 4 or Misc::bad_args(4, scalar @_);
	my $c = shift;
	my $dir = shift;
	my $info_ref = shift;
	my $error_ref = shift;

	my $rball = new Rball;
	if($c->param('email') ne $c->param('old_email'))
	{
		my %info = $rball->get_info($c->param('email'));
		if(keys %info > 0)
		{
			push @$error_ref, "Email address already in use: " . $c->param('email') . "\n";
			return;
		}
	}

	# irrelevant things from $c->param will be ignored...
	eval { $rball->check_user($c->param('email'), map { $_ => $c->param($_) } $c->param) };
	if($@ and $@ =~ /^$Global::error/)
	{
		push @{ $error_ref }, $@;
		return;
	}
	elsif($@)
	{
		die $@;
	}

	my $new_email = $c->param('email');
	my $old_email = $c->param('old_email');
	my $new_ladder = $c->param('ladder');
	my $old_ladder = $c->param('old_ladder');

	$c->delete('old_email');
	$c->delete('old_ladder');
	$c->delete('passwd2');
	$c->delete('Submit modifications');

	print $c->h1("Updates");
	my $text = "";
	foreach($c->param)
	{
		if(/passwd/)
		{
			$text .= "password: " 
				. ("x" x length $c->param($_))
				. $c->br;
			next;
		}
		$text .= "$_: " 
			. $c->param($_) 
			. $c->br;
	}
	print $c->blockquote($text);

	$c->delete('email');
	$c->delete('ladder');

	my %ladder = ();
	if(defined $new_ladder and $new_ladder != $old_ladder)
	{
		%ladder = (
			'ladder' => $new_ladder,
			'rank' => 1 + $rball->max($new_ladder)
		);
	}

	$rball->modify_user(
		$old_email,
		defined $new_email ? $new_email : $old_email,
		%ladder,
		map { $_ => $c->param($_) } $c->param
	);

	if(defined $new_ladder and $old_ladder != $new_ladder)
	{
		$rball->make_ladders(1);
	}


	Misc::debug_end($debug) if defined $debug and $debug;
}

sub process_login_input
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 4 or Misc::bad_args(4, scalar @_);
	my $c = shift;
	my $dir = shift;
	my $info_ref = shift;
	my $error_ref = shift;

	push @$error_ref, "Email address not specified.\n" if ! $c->param('email');
	push @$error_ref, "Password not specified.\n" if ! $c->param('passwd');
	return if @$error_ref > 0;

	my $rball = new Rball;
	%{ $info_ref } = $rball->get_info($c->param('email'));

	push @$error_ref, "Invalid password\n" if ${ $info_ref }{'passwd'} ne $c->param('passwd');

	return;
}

sub print_change_prompt
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ >= 1 or Misc::bad_args("1+", scalar @_);
	my $c = shift;
	my %info = @_;

	print $c->h1($g_page_name),
		$c->startform({-Method => 'POST', -Action => $c->self_url}),
		'Full Name: ',
		$c->textfield(
			-name => "name", 
			-default => $info{'name'}, 
			-size => 30, -maxlength => 50
		),
		$c->br, "\n",
		'Email Address: ',
		$c->textfield(
			-name => "email", 
			-default => $info{'email'},
			-size => 30, 
			-maxlength => 100
		),
		$c->br, "\n",
		'Home Page (optional): ',
		$c->textfield(
			-name => "url", 
			-default => $info{'url'},
			-size => 60, 
			-maxlength => 150, 
		),
		$c->br, "\n",
		'Phone (optional): ',
		$c->textfield(
			-name => "phone", 
			-default => (exists $info{'phone'} ? $info{'phone'} : ""),
			-size => 15, 
			-maxlength => 15
		),
		$c->br, "\n",
		'Affiliation: ',
		$c->popup_menu(
			-name => "affiliation", 
			-Values => ['grad student', 'student', 'faculty', 'staff', 'other'],
			-default => $info{'affiliation'}
		),
		$c->br, "\n",
		'Department or Program (optional): ',
		$c->textfield(
			-name => "department", 
			-default => $info{'department'},
			-size => 30, 
			-maxlength => 50
		),
		$c->br, "\n",
		'Estimated skill level: ',
		$c->radio_group(
			-name => 'ladder',
			-Values => [1, 2, 3],
			-default => $info{'ladder'},
			-labels => {1 => ' Advanced ', 2 => ' Intermediate ', 3 => ' Beginner '}
		),
		$c->br, "\n",
		'Choose a Password: ',
		$c->password_field(
			-name => 'passwd', 
			-default => $info{'passwd'}, # shouldn't be needed...
			-size => 20, 
			-maxlength => 20
		),
		$c->br, "\n",
		'Retype your Password: ',
		$c->password_field(
			-name => 'passwd2', 
			-default => $info{'passwd'},
			-size => 20, 
			-maxlength => 20
		),
		$c->br, "\n",
		$c->hidden(
			-name => 'mode',
			-Value => 'changes'
		),
		$c->br, "\n",
		$c->hidden(
			-name => 'old_email',
			-Value => $c->param('email')
		),
		$c->br, "\n",
		$c->hidden(
			-name => 'old_ladder',
			-Value => $info{'ladder'}
		),
		$c->br, "\n",
		$c->submit(-name => 'Submit modifications'),
		$c->endform;


}


sub print_login_prompt
{
	Misc::debug_start($debug) if defined $debug and $debug;
	@_ == 1 or Misc::bad_args(1, scalar @_);
	my $c = shift;

	my $rball = new Rball;
	my @users = sort $rball->get_email;

	print $c->h1($g_page_name),
		$c->startform({-Method => 'POST', -Action => $c->self_url}),
		'Email Address: ',
		$c->popup_menu(
			-name => "email", 
			-Values => [ @users ],
		),
		$c->br, "\n",
		'Password: ',
		$c->password_field(-name => 'passwd', -size => 20, -maxlength => 20),
		$c->br, "\n",
		$c->hidden(-name => 'mode', -value => 'initial'),
		$c->submit,
		$c->endform;
}

